@@ -18,7 +18,7 @@ from ruamel.yaml.nodes import *  # NOQA
 
 from ruamel.yaml.loader import BaseLoader, SafeLoader, Loader, RoundTripLoader  # NOQA
 from ruamel.yaml.dumper import BaseDumper, SafeDumper, Dumper, RoundTripDumper  # NOQA
-from ruamel.yaml.compat import StringIO, BytesIO, with_metaclass, PY3
+from ruamel.yaml.compat import StringIO, BytesIO, with_metaclass, PY3, nprint
 from ruamel.yaml.resolver import VersionedResolver, Resolver  # NOQA
 from ruamel.yaml.representer import (
     BaseRepresenter,
@@ -146,6 +146,9 @@ class YAML(object):
         self.map_indent = None
         self.sequence_indent = None
         self.sequence_dash_offset = 0
+        self.compact_seq_seq = None
+        self.compact_seq_map = None
+        self.sort_base_mapping_type_on_output = None  # default: sort
 
         self.top_level_colon_align = None
         self.prefix_colon = None
@@ -158,22 +161,26 @@ class YAML(object):
         self.tags = None
         self.default_style = None
         self.top_level_block_style_scalar_no_indent_error_1_1 = False
+        # [a, b: 1, c: {d: 2}]  vs. [a, {b: 1}, {c: {d: 2}}]
+        self.brace_single_entry_mapping_in_flow_sequence = False
 
     @property
     def reader(self):
         # type: () -> Any
-        attr = '_' + sys._getframe().f_code.co_name
-        if not hasattr(self, attr):
-            setattr(self, attr, self.Reader(None, loader=self))
-        return getattr(self, attr)
+        try:
+            return self._reader  # type: ignore
+        except AttributeError:
+            self._reader = self.Reader(None, loader=self)
+            return self._reader
 
     @property
     def scanner(self):
         # type: () -> Any
-        attr = '_' + sys._getframe().f_code.co_name
-        if not hasattr(self, attr):
-            setattr(self, attr, self.Scanner(loader=self))
-        return getattr(self, attr)
+        try:
+            return self._scanner  # type: ignore
+        except AttributeError:
+            self._scanner = self.Scanner(loader=self)
+            return self._scanner
 
     @property
     def parser(self):
@@ -193,7 +200,7 @@ class YAML(object):
                     # else:
                     setattr(self, attr, CParser(self._stream))
                     # self._parser = self._composer = self
-                    # print('scanner', self.loader.scanner)
+                    # nprint('scanner', self.loader.scanner)
 
         return getattr(self, attr)
 
@@ -237,6 +244,7 @@ class YAML(object):
                     allow_unicode=self.allow_unicode,
                     line_break=self.line_break,
                     prefix_colon=self.prefix_colon,
+                    brace_single_entry_mapping_in_flow_sequence=self.brace_single_entry_mapping_in_flow_sequence,  # NOQA
                     dumper=self,
                 )
                 setattr(self, attr, _emitter)
@@ -247,6 +255,10 @@ class YAML(object):
                 if self.sequence_dash_offset is not None:
                     _emitter.sequence_dash_offset = self.sequence_dash_offset
                     # _emitter.block_seq_indent = self.sequence_dash_offset
+                if self.compact_seq_seq is not None:
+                    _emitter.compact_seq_seq = self.compact_seq_seq
+                if self.compact_seq_map is not None:
+                    _emitter.compact_seq_map = self.compact_seq_map
             else:
                 if getattr(self, '_stream', None) is None:
                     # wait for the stream
@@ -278,15 +290,14 @@ class YAML(object):
         # type: () -> Any
         attr = '_' + sys._getframe().f_code.co_name
         if not hasattr(self, attr):
-            setattr(
-                self,
-                attr,
-                self.Representer(
-                    default_style=self.default_style,
-                    default_flow_style=self.default_flow_style,
-                    dumper=self,
-                ),
+            repres = self.Representer(
+                default_style=self.default_style,
+                default_flow_style=self.default_flow_style,
+                dumper=self,
             )
+            if self.sort_base_mapping_type_on_output is not None:
+                repres.sort_base_mapping_type_on_output = self.sort_base_mapping_type_on_output
+            setattr(self, attr, repres)
         return getattr(self, attr)
 
     # separate output resolver?
@@ -321,11 +332,11 @@ class YAML(object):
         finally:
             parser.dispose()
             try:
-                self._reader.reset_reader()  # type: ignore
+                self._reader.reset_reader()
             except AttributeError:
                 pass
             try:
-                self._scanner.reset_scanner()  # type: ignore
+                self._scanner.reset_scanner()
             except AttributeError:
                 pass
 
@@ -353,11 +364,11 @@ class YAML(object):
         finally:
             parser.dispose()
             try:
-                self._reader.reset_reader()  # type: ignore
+                self._reader.reset_reader()
             except AttributeError:
                 pass
             try:
-                self._scanner.reset_scanner()  # type: ignore
+                self._scanner.reset_scanner()
             except AttributeError:
                 pass
 
@@ -389,17 +400,17 @@ class YAML(object):
                 # if you just initialise the CParser, to much of resolver.py
                 # is actually used
                 rslvr = self.Resolver
-                if rslvr is ruamel.yaml.resolver.VersionedResolver:
-                    rslvr = ruamel.yaml.resolver.Resolver
+                # if rslvr is ruamel.yaml.resolver.VersionedResolver:
+                #     rslvr = ruamel.yaml.resolver.Resolver
 
                 class XLoader(self.Parser, self.Constructor, rslvr):  # type: ignore
-                    def __init__(selfx, stream, version=None, preserve_quotes=None):
+                    def __init__(selfx, stream, version=self.version, preserve_quotes=None):
                         # type: (StreamTextType, Optional[VersionType], Optional[bool]) -> None  # NOQA
                         CParser.__init__(selfx, stream)
                         selfx._parser = selfx._composer = selfx
                         self.Constructor.__init__(selfx, loader=selfx)
                         selfx.allow_duplicate_keys = self.allow_duplicate_keys
-                        rslvr.__init__(selfx, loadumper=selfx)
+                        rslvr.__init__(selfx, version=version, loadumper=selfx)
 
                 self._stream = stream
                 loader = XLoader(stream)
@@ -451,7 +462,7 @@ class YAML(object):
         """
         if not hasattr(stream, 'write') and hasattr(stream, 'open'):
             # pathlib.Path() instance
-            with stream.open('w') as fp:  # type: ignore
+            with stream.open('w') as fp:
                 return self.dump_all(documents, fp, _kw, transform=transform)
         if _kw is not enforce:
             raise TypeError(
@@ -478,7 +489,7 @@ class YAML(object):
                 try:
                     self.representer.represent(data)
                 except AttributeError:
-                    # print(dir(dumper._representer))
+                    # nprint(dir(dumper._representer))
                     raise
             self.serializer.close()
         finally:
@@ -490,7 +501,7 @@ class YAML(object):
             delattr(self, '_serializer')
             delattr(self, '_emitter')
         if transform:
-            val = stream.getvalue()  # type: ignore
+            val = stream.getvalue()
             if self.encoding:
                 val = val.decode(self.encoding)
             if fstream is None:
@@ -651,11 +662,11 @@ class YAML(object):
         finally:
             parser.dispose()
             try:
-                self._reader.reset_reader()  # type: ignore
+                self._reader.reset_reader()
             except AttributeError:
                 pass
             try:
-                self._scanner.reset_scanner()  # type: ignore
+                self._scanner.reset_scanner()
             except AttributeError:
                 pass
 
@@ -669,7 +680,7 @@ class YAML(object):
     def __exit__(self, typ, value, traceback):
         # type: (Any, Any, Any) -> None
         if typ:
-            print('typ', typ)
+            nprint('typ', typ)
         self._context_manager.teardown_output()
         # self._context_manager.teardown_input()
         self._context_manager = None
@@ -704,10 +715,15 @@ class YAML(object):
         # type: (Any) -> None
         self.sequence_dash_offset = val
 
+    def compact(self, seq_seq=None, seq_map=None):
+        # type: (Any, Any) -> None
+        self.compact_seq_seq = seq_seq
+        self.compact_seq_map = seq_map
+
 
 class YAMLContextManager(object):
     def __init__(self, yaml, transform=None):
-        # type: (Any, Optional[Callable]) -> None
+        # type: (Any, Any) -> None  # used to be: (Any, Optional[Callable]) -> None
         self._yaml = yaml
         self._output_inited = False
         self._output_path = None
@@ -784,7 +800,7 @@ class YAMLContextManager(object):
         try:
             self._yaml.representer.represent(data)
         except AttributeError:
-            # print(dir(dumper._representer))
+            # nprint(dir(dumper._representer))
             raise
 
     # def teardown_input(self):
@@ -1159,7 +1175,7 @@ def dump_all(
             try:
                 dumper._representer.represent(data)
             except AttributeError:
-                # print(dir(dumper._representer))
+                # nprint(dir(dumper._representer))
                 raise
         dumper._serializer.close()
     finally:
